//
//	interpolator.cc: test the Interpolator class
//

#include <cassert>
#include <cmath>
#include "Interpolator.h"

//
//	f(x) = 3*x^4 - 1.5 * x^3 + x^2 - 3.5 * x +8
//
static double f4(double x)
{
	double x2 = x * x, x3 = x2 * x, x4 = x2 * x2;

	return 3*x4 - 1.5*x3 + x2 - 3.5 * x +8;
}


static bool isClose(double a, double b)
{
	return fabs(a)/fabs(b)-1. < 1.e-8;
}

int main()
{
	double xp[4] = { -10., -5., 1., 3. },
		   yp[4][3];

	for (unsigned i=0; i<3; ++i)
		for (unsigned j=0; j<4; ++j)
			yp[j][i] = (i+1)*f4(xp[j]);

	double y[3];
	Interpolator<3>::doPoly4(xp, yp, xp[0], y);
	assert(isClose(y[0], yp[0][0]) && isClose(y[1], yp[0][1]) &&
		   isClose(y[2], yp[0][2]));

	double arg = 37.3;
	Interpolator<3>::doPoly4(xp, yp, arg, y);
	assert(isClose(y[0], f4(arg)) && isClose(y[1], 2.*f4(arg)) &&
		   isClose(y[2], f4(arg)));

	return 0;
}
