//
//	oalmisc.cc: test miscelaneous OAL functionality
//
//	DESCRIPTION:
//		This test program checks miscellaneous OAL functionality.
//
//	CHANGE HISTORY:
//		UL, 1999-02-02: 1.0.0: - first version
//		
//	AUTHOR:
//		U. Lammers, Tue Feb  2 14:38:37 MET 1999
//

#include <iostream>
#include <cassert>
#include <cmath>
#include "errstr.h"
#include "OalServer.h"
#include "Celestial.h"
#include "Xmm.h"

int nearestInt(double x)
{
	return INT(x<0. ? ceil(x-.5) : floor(x+.5));
}

void checkAngle(double rad, int hundredthArcsec)
{
	assert(nearestInt(rad*RAD_TO_DEG*3600.*100.) == hundredthArcsec);
}

int main()
{
	const Odf *odf = oalServer.newOdf("../odf");

	assert(odf->expandSetName("xxx") == "xxx");
	cout<<odf->expandSetName("odf(emos1, ima, 1, 0)")<<endl;
//	assert(odf->expandFileName("odf(emos1, ima, 1, 0)")
//				== "../odf/0010_PPPPPPOOLL_M1S00110IME.FIT");


	checkAngle(Declination("-0:00:00.00").angle(), 0);
	checkAngle(Declination("-0:30:00.00").angle(), -180000);
	checkAngle(Declination("0:30:00.00").angle(), 180000);
	checkAngle(Declination("-0:59:59.99").angle(), -359999);
	checkAngle(Declination("0:59:59.99").angle(), 359999);
	checkAngle(Declination("-1:00:00.00").angle(), -360000);
	checkAngle(Declination("180.").angle(), 64800000);
	checkAngle(Declination("-180.").angle(), -64800000);
	checkAngle(RightAscension("180.").angle(), 64800000);
	checkAngle(RightAscension("-180.").angle(), -64800000);

	cout<<"\nAll tests successfully passed.\n";

	return 0;
}
