//
//	tags.cc: test the Tags classes
//
//	DESCRIPTION:
//		test the Tags classes by converting a number of frame counters
//		to proper time tags - the frames in the dummy ODF EMOS1 auxiliary
//		file are taken for taken for that purpose
//
//	CHANGE HISTORY:
//		UL, 1998-08-19: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers, Wed Aug 19 09:41:52 MET DST 1998
//

#include <iostream>
#include <cassert>
#include <cmath>
#include "errstr.h"
#include "TimeTags.h"
#include "OalServer.h"

int main()
{
	//
	//	read the frame numbers in the dummy EMOS1 auxiliary file
	//
	oalServer.instrument(Xmm::EMOS2);
	oalServer.exposureIndex(1);
	int32 v[] = { 15, 16, 17, 18, 19 };
	const FrameCounterVector *frameVec = oalServer.newFrameCounterVector(
											v, NUM_OF(v));

	ObtVector obtvec(frameVec->toObt());

	vector<double> metVec(obtvec.toMET());

	cout<<"Frame counters - OBT - tag - time:\n";
	cout.precision(16);
	for (unsigned n=0; n<metVec.size(); ++n) {
		STime t = Xmm::missionReferenceTime + metVec[n];
		cout<<v[n]<<" "<<obtvec[n]<<" "<<metVec[n]<<" "
			<<t.toString()<<endl;
	}
	return 0;
}
