//
//	auxhkgen_emos.cc: create EPN auxiliary + periodic HK files
//
//	Create a simple auxiliary file with frame counters and OBT tags
//	for EPN - also create a simple version of EPN main periodic HK file
//	with the EPIC clock reset time at the start	of the exposure #1 in
//	specified in the summary file.
//
//	This is to test the Tags classes. Also create two dummy parameter
//	columns in main periodic HK files (P1_MAIN (sin wave) + P2_MAIN
//	(cos wave)); create Additional Periodic HK file with one parameter
//	P3_ADD + time axis column TIME_D; this is to test task hkgtigen
//
#include "common.h"
#include "Dal.h"
#include "STime.h"
#include "CalOalUtility.h"
#include "Celestial.h"

using namespace CalOalUtility;

STime start("1999-08-28T08:05:00");

#define NUM_OF(a)	(sizeof(a)/sizeof(a[0]))
int main()
{
	static const unsigned short coarseWrap = 32400;

	int16 coarset[] = { 0, coarseWrap-2, coarseWrap-1, 0,1, 1, 1, 2, 2, 3, 3 };
	uint16  finet[] = {	0, 0,		23,	   999, 1234, 3333, 9999, 12345,
						24999, 17889, 17890 };

	//	write auxiliary file 
	DataSet *set = dataSetServer->open("0010_PPPPPPOOLL_PNS00100AUX.FIT",
									   DataSet::Create);
	Table *tab = set->addTable("AUXILIARY", 12*NUM_OF(coarset));
	ColumnData *cdframe = tab->addColumn("FRAME", Dal::Int32,
										 "frame counter", "")->data();
	int32 *frame = cdframe->int32Data();

	ColumnData *cdccdid = tab->addColumn("CCDID", Dal::Int8,
										 "CCD identifier", "")->data(),
			   *cdquadrant = tab->addColumn("QUADRANT", Dal::Int8,
										 "quadrant id", "")->data();
	int8  *ccdid = cdccdid->int8Data(),
		  *quadrant = cdquadrant->int8Data();

	ColumnData *cdftcoarse = tab->addColumn("FTCOARSE", Dal::Int16,
								"Coarse Frame Time", "s")->data(),
			   *cdftfine = tab->addColumn("FTFINE", Dal::Uint16,
								"Fine Frame Time", "40usec")->data();

	int16 *ftcoarse = cdftcoarse->int16Data();
	uint16 *ftfine = cdftfine->uint16Data();

	unsigned nn=0;
	for (unsigned quad=0; quad<4; ++quad)
		for (unsigned ccd=0; ccd<3; ++ccd)
			for (unsigned n=0; n<NUM_OF(coarset); ++n) {
				frame[nn] = n;
				ccdid[nn] = ccd;
				quadrant[nn] = quad;
				ftcoarse[nn] = coarset[n];
				ftfine[nn] = finet[n];
				++nn;
		}

	//	tidy up
	delete cdftcoarse;
	delete cdftfine;
	delete cdframe;
	delete cdccdid;
	delete cdquadrant;
	dataSetServer->close(set);

	//
	//	write periodic HK file
	//
	set = dataSetServer->open("0010_PPPPPPOOLL_PNS00100PMH.FIT",
							   DataSet::Create);
	tab = set->addTable("MAIN_PERIODIC_HK", 101);

    vector<unsigned long> size; size.push_back(19);
    ColumnData *tcol = tab->addColumn("TIME", Dal::DString,
                                     "time tag", "ERT", size)->data();

	ColumnData *cdcoarseobt = tab->addColumn("F1030", Dal::Int32,
									"COBT coarse", "sec")->data(),
			   *cdfineobt = tab->addColumn("FU036", Dal::Uint16,
									"COBT fine", "1/65536s")->data(),
			   *cdcoarsewrap = tab->addColumn("F1052", Dal::Uint16,
								    "COBT coarse wrap value", "s")->data(),
			   *cdp1 = tab->addColumn("P1_MAIN", Dal::Real32, "P1")->data(),
			   *cdp2 = tab->addColumn("P2_MAIN", Dal::Real32, "P2")->data(),
			   *cdtime = tab->addColumn("TIME_D", Dal::Real64,
										"time coordinate")->data();
	int32 *coarseobt = cdcoarseobt->int32Data();
	uint16 *fineobt = cdfineobt->uint16Data(),
		   *coarsewrap = cdcoarsewrap->uint16Data();
	real32 *p1 = cdp1->real32Data(),
		   *p2 = cdp2->real32Data();
	real64 *time = cdtime->real64Data();

	for (unsigned n=0; n<tab->rows(); ++n) {
		time[n] = DBL(n);
		STime t = start + 10 * n;
		stringColumnRowSet(tcol, n, t.toString());
		coarsewrap[n] = coarseWrap;
		coarseobt[n] = 1<<31;
		fineobt[n] = 21845;
		double arg = DBL(n)/DBL(tab->rows()-1)*2.*M_PI;
		p1[n] = sin(arg);
		p2[n] = cos(arg);
	}

	delete cdtime;
	delete tcol;
	delete cdcoarseobt;
	delete cdfineobt;
	delete cdcoarsewrap;
	delete cdp1;
	delete cdp2;
	dataSetServer->close(set);

	//
	//	write additional periodic HK file
	//
	set = dataSetServer->open("0010_PPPPPPOOLL_PNS00100PAH.FIT",
							   DataSet::Create);

	tab = set->addTable("ADDITIONAL_PERIODIC_HK", 101);
	tcol = tab->addColumn("TIME", Dal::DString,
                                     "time tag", "ERT", size)->data();
	cdtime = tab->addColumn("TIME_D", Dal::Real64,
							"time coordinate")->data();

	ColumnData *cdp3 = tab->addColumn("P3_ADD", Dal::Real32, "P3")->data();
	real32 *p3 = cdp3->real32Data();
	time = cdtime->real64Data();

	for (unsigned n=0; n<tab->rows(); ++n) {
		time[n] = DBL(n);
		STime t = start + 10 * n;
		stringColumnRowSet(tcol, n, t.toString());
		double arg = DBL(n)/DBL(tab->rows()-1)*2.*M_PI;
		p3[n] = sin(arg);
	}

	delete cdtime;
	delete tcol;
	delete cdp3;


	dataSetServer->close(set);
}
