//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	AttitudeLocator.h	--- classes streamlined for S/C Attitude retrieval
//
//	DESCRIPTION:
//		A class AttitudeLocator + auxiliary classes are defined wich
//		provide access to the S/C attitude information on each ODF.
//
//	CHANGE HISTORY:
//		UL, 1998-08-11: 1.0.0: - first version
//		UL, 1999-01-03: 1.1.0: - changed from position angle to astronomical
//								 position angle
//							   - added SpacecraftAttitude::toEulerAngles()
//		UL, 1999-08-18: 1.2.0: - added exception class
//		UL, 2001-04-02: 1.3.0: - added constructFromRAF()
//									
//
//	AUTHOR:
//		U. Lammers, Tue Aug 11 15:55:34 MET DST 1998
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//		[2]: ICD: Observation and Slew Data Files (ODF-ICD), K. Galloway
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/oal/src/AttitudeLocator.h,v 1.11 2001/07/03 13:26:07 ulammers Exp $
//

#ifndef ATTITUDE_LOCATOR_H
#define ATTITUDE_LOCATOR_H

#include "Tracer.h"
#include "OalChild.h"
#include "Odf.h"
#include "TimeAxis.h"
#include "Celestial.h"

class OalServer;

class AttitudeLocator : public virtual TimeAxis,
						public virtual OalChild<OalServer> {
public:
	//
	//	exception class: to be thrown if no attitude data is available
	//	at requested time
	//
	class NoAttitudeDataAvailable {};

	AttitudeLocator();
	AttitudeLocator(const OalServer *oalEngine,
					double maxDelta = 10.);

	static const double granularity = 1.;	//	nominal data granularity

	SpacecraftAttitude attitude(double time) const;

	~AttitudeLocator();

private:
	//	data
	vector<double>	_ra, _dec, _apos;
	double			_maxDelta;

protected:
	//	methods
	void constructFromAHF(const string &, const STime &, const STime &);
	void constructFromRAF(const string &, const STime &, const STime &);
	void constructFromTHF(const string &, const STime &, const STime &);
};

#endif



