//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Interpolator.cc	--- implementation of polynomial interpolation
//
//	DESCRIPTION:
//		Polynomial interpolation - implementation
//
//	CHANGE HISTORY:
//		UL, 1998-08-11: 1.0.0: - first version
//		UL, 1999-04-28: 1.1.0: - fatal -> error
//
//	AUTHOR:
//		U. Lammers, Tue Aug 11 15:55:34 MET DST 1998
//
//	REFERENCES:
//		[1]: Numerical Recipes in C, Second Edition, W.H. Press et. al.
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/oal/src/Interpolator.cc,v 1.6 2000/04/19 14:20:17 gvacanti Exp $
//

#include "Interpolator.h"
#include "errstr.h"

//
//	construct 3 4th order polynomials and evaluate them at given point
//	straight Lagrange-formula implementation
//
template<unsigned N>
void Interpolator<N>::doPoly4(double xp[4], double yp[4][N], double x,
							  double y[N])
{
	PROC

	double      ttx[4], txty[4][N];

	for (unsigned i=0; i<4; ++i) {
		unsigned    k;

		ttx[i] = x - xp[i];
		k = 0;
		for (unsigned j=0; j<N; ++j) {
			txty[i][j] = xp[i] - xp[i!=j ? k : ++k];
			if (txty[i][j] == 0.)
				errstr<<"Two points are identical - cannot construct "
						"a polynomial"
					 <<Sas::error("IllegalValues");
			++k;
		}
	}

	for (unsigned i=0; i<N; ++i)
		y[i] =
				(ttx[1]*ttx[2]*ttx[3])/(txty[0][0]*txty[0][1]*txty[0][2]) *
					yp[0][i] +
				(ttx[0]*ttx[2]*ttx[3])/(txty[1][0]*txty[1][1]*txty[1][2]) *
					yp[1][i] +
				(ttx[0]*ttx[1]*ttx[3])/(txty[2][0]*txty[2][1]*txty[2][2]) *
					yp[2][i] +
				(ttx[0]*ttx[1]*ttx[2])/(txty[3][0]*txty[3][1]*txty[3][2]) *
					yp[3][i];

	ENDPROC
} // doPoly4<N>

template Interpolator<3>;
template Interpolator<6>;
