//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Interpolator.h	--- polynomial interpolation
//
//	DESCRIPTION:
//		Polynomial interpolation
//
//	CHANGE HISTORY:
//		UL, 1998-08-11: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers, Tue Aug 11 15:55:34 MET DST 1998
//
//	REFERENCES:
//		[1]: Numerical Recipes in C, Second Edition, W.H. Press et. al.
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/oal/src/Interpolator.h,v 1.4 2000/04/19 14:20:17 gvacanti Exp $
//

#ifndef INTERPOLATOR_H
#define INTERPOLATOR_H

#include "Tracer.h"

template<unsigned N>
class Interpolator {
public:
//	move the template inside the class when we have templated member functions

	Interpolator() { PROC ENDPROC }

	static void doPoly4(double xp[4], double yp[4][N], double x, double y[N]);
		//
		//	PURPOSE:
		//		Interpolation with a 4th order polynomials
		//
		//	ARGUMENTS:
		//		I:	xp[4]	: vector with coordinates of known points
		//			yp[4][N]: N sets of values of the polynomials at the <xp>'s
		//			x		: coordinate at which the polynomials are to be
		//					  evaluated
		//		O:	y[N]	: values of the polynomials at point <x>
		//

	~Interpolator() { PROC ENDPROC }
};

#endif



