//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	TimeTags.h	--- classes for CCD frame counters and time tags
//
//	DESCRIPTION:
//		Classes representing CCF frame counters and time tags are defined.
//
//	CHANGE HISTORY:
//		UL, 1998-08-03: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers, Mon Aug  3 14:31:48 MET DST 1998
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/oal/src/TimeTags.h,v 1.1 2001/04/02 11:46:05 ulammers Exp $
//

#ifndef TIME_TAGS_H
#define TIME_TAGS_H

#include "Dal.h"
#include "Tracer.h"
#include "OalChild.h"
#include "Xmm.h"

#include <vector>

class OalServer;
class ObtVector;

class FrameCounterVector : public virtual OalChild<OalServer> {
public:
	FrameCounterVector(const OalServer *oalEngine = 0)
		: OalChild<OalServer>(oalEngine) { PROC ENDPROC }

	virtual ObtVector toObt() const = 0;

	virtual ~FrameCounterVector() {}
};


template<class Instr>
class InstrFrameCounterVector : public FrameCounterVector,
	public vector<int32> {
public:
	InstrFrameCounterVector(const int32 counterVec[], unsigned nCounter,
							const OalServer *oalEngine = 0)
		: OalChild<OalServer>(oalEngine), FrameCounterVector(oalEngine)
	{
		this->reserve(nCounter);
		for (unsigned n=0; n<nCounter; this->push_back(counterVec[n++]));
	}

	static void resetWraps() { PROC _nWraps = 0; }
	static void resetResetTime() { PROC _resetTime = -1.; ENDPROC }
	static void reset() { PROC resetWraps(); resetResetTime(); ENDPROC }

	ObtVector toObt() const;

	~InstrFrameCounterVector() {}

private:
	//
	//	data
	//
	static unsigned _nWraps;		//	number of instrument clock
									//	wrap-arounds
	static double	_resetTime,		//	time of instrument clock reset
					_wrapValue;		//	value at which coarse time wraps
									//	around

	//
	//	methods
	//
	void setStatics() const;		//	check if reset time/warp value
									//	are defined
};



class ObtVector : public virtual OalChild<FrameCounterVector>,
	public vector<double> {
public:
	ObtVector(const FrameCounterVector *frameCounterVector = 0)
		: OalChild<FrameCounterVector>(frameCounterVector) { PROC ENDPROC }

	vector<double> toMET() const;

	~ObtVector() {}
};

#endif
